Struct OccupiedEntry

pub struct OccupiedEntry<'a, K> { /* private fields */ }
A representation of a qualifier that exists in the list.

Implementations

Source

impl<'a, K> OccupiedEntry<'a, K>

pub fn remove_entry(self) -> (SmallString, SmallString)

Remove the qualifier from the list and return it as a key-value pair.

pub fn get(&self) -> &str

Get the value of the qualifier.

pub fn get_mut(&mut self) -> &mut SmallString

Get the value of the qualifier.

pub fn into_mut(self) -> &'a mut SmallString

Convert this entry into a mutable reference to the qualifier’s value.

This is similar to Self::get_mut() but has different lifetimes.

pub fn insert<V>(&mut self, value: V) -> SmallString
where SmallString: From<V>,

Overwrite the value of the qualifier.

The previous value is returned.

pub fn remove(self) -> SmallString

Remove the qualifier from the list and return its value.