pub struct OccupiedEntry<'a, K> { /* private fields */ }Expand description
A representation of a qualifier that exists in the list.
Implementations
Sourceimpl<'a, K> OccupiedEntry<'a, K>
impl<'a, K> OccupiedEntry<'a, K>
Sourcepub fn remove_entry(self) -> (SmallString, SmallString)
pub fn remove_entry(self) -> (SmallString, SmallString)
Remove the qualifier from the list and return it as a key-value pair.
Sourcepub fn get_mut(&mut self) -> &mut SmallString
pub fn get_mut(&mut self) -> &mut SmallString
Get the value of the qualifier.
Sourcepub fn into_mut(self) -> &'a mut SmallString
pub fn into_mut(self) -> &'a mut SmallString
Convert this entry into a mutable reference to the qualifier’s value.
This is similar to Self::get_mut() but has different lifetimes.
Sourcepub fn insert<V>(&mut self, value: V) -> SmallStringwhere
SmallString: From<V>,
pub fn insert<V>(&mut self, value: V) -> SmallStringwhere
SmallString: From<V>,
Overwrite the value of the qualifier.
The previous value is returned.
Sourcepub fn remove(self) -> SmallString
pub fn remove(self) -> SmallString
Remove the qualifier from the list and return its value.